package com.tuoheng.common.core.utils;

import org.apache.commons.lang3.time.DateFormatUtils;

import java.lang.management.ManagementFactory;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * 时间工具类
 */
public final class DateUtils extends org.apache.commons.lang3.time.DateUtils {

    public static String YYYY = "yyyy";

    public static String YYYY_MM = "yyyy-MM";

    public static String YYYY_MM_DD = "yyyy-MM-dd";

    public static String YYYYMMDD = "yyyyMMdd";
    public static String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";

    public static String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    public static String HHMMSS = "HH:mm:ss";

    private static String[] parsePatterns = {
            "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM",
            "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
            "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM"};


    /**
     * 获取当前时间
     *
     * @return
     */
    public static Date now() {
        return new Date();
    }

    /**
     * 获取当前日期, 默认格式为yyyy-MM-dd
     *
     * @return String
     */
    public static String getDate() {
        return dateTimeNow(YYYY_MM_DD);
    }

    public static final String getTime() {
        return dateTimeNow(YYYY_MM_DD_HH_MM_SS);
    }

    public static final String dateTimeNow() {
        return dateTimeNow(YYYYMMDDHHMMSS);
    }

    public static final String dateTimeNow(final String format) {
        return parseDateToStr(format, new Date());
    }

    public static final String dateTime(final Date date) {
        return parseDateToStr(YYYY_MM_DD, date);
    }

    public static final String parseDateToStr(final String format, final Date date) {
        return new SimpleDateFormat(format).format(date);
    }

    public static final Date dateTime(final String format, final String ts) {
        try {
            return new SimpleDateFormat(format).parse(ts);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 检查字符串是否使时间格式
     *
     * @param format 格式
     * @param ts     时间字符串
     * @return 布尔值
     */
    public static boolean checkStrIsDate(final String format, final String ts) {
        try {
            new SimpleDateFormat(format).parse(ts);
            return true;
        } catch (ParseException e) {
            return false;
        }
    }

    /**
     * 日期路径 即年/月/日 如2018/08/08
     */
    public static final String datePath() {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyy/MM/dd");
    }

    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String dateTime() {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyyMMdd");
    }

    /**
     * 日期型字符串转化为日期 格式
     */
    public static Date parseDate(Object str) {
        if (str == null) {
            return null;
        }
        try {
            return parseDate(str.toString(), parsePatterns);
        } catch (ParseException e) {
            return null;
        }
    }

    /**
     * 获取服务器启动时间
     */
    public static Date getServerStartDate() {
        long time = ManagementFactory.getRuntimeMXBean().getStartTime();
        return new Date(time);
    }

    /**
     * 计算两个时间差
     */
    public static String getDatePoor(Date endDate, Date nowDate) {
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        // long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - nowDate.getTime();
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        // long sec = diff % nd % nh % nm / ns;
        return day + "天" + hour + "小时" + min + "分钟";
    }

    /**
     * 根据时间生成code
     */
    public static String generateCode(String prefix) {
        // 根据当前时间，生成任务编码
        Date date = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss");
        return prefix + formatter.format(date);
    }

    /**
     * 根据时间生成code
     */
    public static String generateCode(String prefix, String format, String random) {
        // 根据当前时间，生成任务编码
        Date date = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat(format);
        return prefix + formatter.format(date) + random;
    }

    /**
     * 生成图片编号，P+年月日时分秒+随机三位数
     *
     * @return
     */
    public static String generateFileCode() {
        String Date = DateUtils.dateTimeNow(DateUtils.YYYYMMDDHHMMSS);
        int randomNum = (int) (Math.random() * 900) + 100;
        String fileCode = "P" + Date + randomNum;

        return fileCode;
    }

    /**
     * 返回字符串
     * <p>
     * 给原本的时间originDate加上自定义的时间
     *
     * @param originDate 原本的时间
     * @param day        要加的天数
     * @param hour       要加的小时数
     * @param minute     要加的分钟数
     * @param second     要加的秒数
     * @return 返回加完时间后的时间str_goalDate
     */
    public static String addDateTimeToStr(Date originDate, int day, int hour, int minute, int second) {
        SimpleDateFormat dateFormate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar cal = Calendar.getInstance();
        cal.setTime(originDate);
        cal.add(Calendar.DATE, day);// 24小时制,加天
        cal.add(Calendar.HOUR, hour);// 24小时制 ,加小时
        cal.add(Calendar.MINUTE, minute);// 24小时制,加分钟
        cal.add(Calendar.SECOND, second);// 24小时制,加秒

        String str_goalDate = dateFormate.format(cal.getTime());
        return str_goalDate;
    }

    /**
     * 返回java.util.Date
     * <p>
     * 给原本的时间originDate加上自定义的时间
     *
     * @param originDate 原本的时间
     * @param day        要加的天数
     * @param hour       要加的小时数
     * @param minute     要加的分钟数
     * @param second     要加的秒数
     * @return 返回加完时间后的时间goalDate
     */
    public static Date addDateTimeToDate(Date originDate, int day, int hour, int minute, int second) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(originDate);
        cal.add(Calendar.DATE, day);// 24小时制,加天
        cal.add(Calendar.HOUR, hour);// 24小时制 ,加小时
        cal.add(Calendar.MINUTE, minute);// 24小时制,加分钟
        cal.add(Calendar.SECOND, second);// 24小时制,加秒

        Date goalDate = cal.getTime();
        return goalDate;
    }

    /**
     * 比较2个时间
     *
     * @param date1
     * @param date2
     * @return -1：时间格式不对
     * 0：date1 == date2
     * 1：date1 < date2
     * 2：date1 > date2
     */
    public static int compareToDate(Date date1, Date date2) {
        String dateStr1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date1);
        String dateStr2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date2);
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar c1 = Calendar.getInstance();
        Calendar c2 = Calendar.getInstance();
        try {
            c1.setTime(df.parse(dateStr1));
            c2.setTime(df.parse(dateStr2));
        } catch (java.text.ParseException e) {
            System.err.println("格式不正确");
            return -1;
        }
        int result = c1.compareTo(c2);
        if (result == 0) {
            return 0;
        } else if (result < 0) {
            return 1;
        } else {
            return 2;
        }
    }

    /**
     * 比较2个时间
     *
     * @param dateStr1
     * @param dateStr2
     * @return -1：时间格式不对
     * 0：date1 == date2
     * 1：date1 < date2
     * 2：date1 > date2
     * -1: 异常
     */
    public static int compareToDate(String dateStr1, String dateStr2) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date date1;
        Date date2;
        try {
            date1 = sdf.parse(dateStr1);
            date2 = sdf.parse(dateStr2);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        if (date1.compareTo(date2) > 0) {
            return 2;
        } else if (date1.compareTo(date2) < 0) {
            return 1;
        } else if (date1.compareTo(date2) == 0) {
            return 0;
        } else {
            return -1;
        }
    }

    /**
     * 计算下一个循环周期
     *
     * @param beginDate 开始日期：格式yyyy-MM-dd HH:mm:ss
     * @param cycleType 周期类型0-每周 1-每月
     * @param cycleDate 循环日期：如果周期类型是每周，那么循环日期日期值范围为：1-7，如果周期类型为每月，那么周期日期值范围为：1-31
     * @return
     * @throws ParseException
     */
    public static String getNextCycleDay(String beginDate, String cycleType, String cycleDate) throws ParseException {
        if (StringUtils.isNotEmpty(cycleType) && "0".equals(cycleType)) {
            if (StringUtils.isNotEmpty(cycleDate) && (Integer.valueOf(cycleDate) < 0 || Integer.valueOf(cycleDate) > 7)) {
                throw new RuntimeException("循环周期类型为周，循环日期只能再1~7范围内！");
            }
        } else if (StringUtils.isNotEmpty(cycleType) && "1".equals(cycleType)) {
            if (StringUtils.isNotEmpty(cycleDate) && (Integer.valueOf(cycleDate) < 0 || Integer.valueOf(cycleDate) > 31)) {
                throw new RuntimeException("循环周期类型为月，循环日期只能再1~31范围内！");
            }
        }

        Date dateBegin = DateUtils.parseDate(beginDate, YYYY_MM_DD_HH_MM_SS);
        String beginDateTime = DateUtils.parseDateToStr(HHMMSS, dateBegin);//开始时间点
        String nowDateTime = DateUtils.parseDateToStr(HHMMSS, DateUtils.now());//现在时间点
        String nextDate = "";//下一个日期
        Date date = null;

        String today = DateUtils.getDate();//获取今天的日期
        //开始日期大于当前，取开始日期进行比较，否则取当天进行比较
        if (beginDate.compareTo(today) > 0) {
            date = DateUtils.parseDate(beginDate, YYYY_MM_DD_HH_MM_SS);
        } else {
            date = new Date();
        }
        if ("0".equals(cycleType)) {
            //每周
            //如果指定周几大于今天所属的周几，则取本周的周几
            //如果指定周几小于今天所属的周几，则取下个周的周几
            Calendar cal = Calendar.getInstance(Locale.CHINA);
            cal.setTime(date);
            cal.setFirstDayOfWeek(Calendar.MONDAY);//将每周第一天设为星期一，默认是星期天
            int week = cal.get(Calendar.DAY_OF_WEEK) - 1;//获取指定日期的周几
            if (week == 0) week = 7;
            if (Integer.valueOf(cycleDate) > week) {//如果指定的周几大于等于当前周几，取本周的周几
                cal.set(Calendar.DAY_OF_WEEK, Integer.valueOf(cycleDate) + 1);//星期
                //如果指定的周几等于当前周几且时间点也大于当前时间点，取本周的周几
            }else if (Integer.valueOf(cycleDate) == week && DateUtils.parseDate(beginDateTime, HHMMSS).after(DateUtils.parseDate(nowDateTime, HHMMSS))) {
                cal.set(Calendar.DAY_OF_WEEK, Integer.valueOf(cycleDate) + 1);//星期
            }else {
                cal.add(Calendar.WEEK_OF_MONTH, 1);//周数加1，即下周
                cal.set(Calendar.DAY_OF_WEEK, Integer.valueOf(cycleDate) + 1);//星期
            }
            nextDate = DateUtils.parseDateToStr(YYYY_MM_DD, cal.getTime());
        } else if ("1".equals(cycleType)) {
            //每月
            if (cycleDate.length() == 1) {
                cycleDate = "0" + cycleDate;
            }
            //如果指定每月几号大于今天所属几号，则取本月的几号
            if (Integer.valueOf(cycleDate) > Integer.valueOf(DateUtils.parseDateToStr("dd", date))) {
                while (true) {
                    Calendar cal = Calendar.getInstance();
                    cal.setTime(date);
                    //如果当月最大天数小于指定的每月几号，则取下一个月的几号，直到找到对应月数有对应的几号为止
                    if (cal.getActualMaximum(Calendar.DATE) < Integer.valueOf(cycleDate)) {
                        date = DateUtils.getLastMonthDate(DateUtils.parseDateToStr(YYYY_MM_DD_HH_MM_SS, date), 1);
                    } else {
                        break;
                    }
                }
                nextDate = DateUtils.parseDateToStr("yyyy-MM", date) + "-" + cycleDate;
                //如果指定每月几号等于今天所属几号且时间点也大于当前时间点，则取本月的几号
            } else if (Integer.valueOf(cycleDate) == Integer.valueOf(DateUtils.parseDateToStr("dd", date)) &&
                    DateUtils.parseDate(beginDateTime, HHMMSS).after(DateUtils.parseDate(nowDateTime, HHMMSS))) {
                while (true) {
                    Calendar cal = Calendar.getInstance();
                    cal.setTime(date);
                    //如果当月最大天数小于指定的每月几号，则取下一个月的几号，直到找到对应月数有对应的几号为止
                    if (cal.getActualMaximum(Calendar.DATE) < Integer.valueOf(cycleDate)) {
                        date = DateUtils.getLastMonthDate(DateUtils.parseDateToStr(YYYY_MM_DD_HH_MM_SS, date), 1);
                    } else {
                        break;
                    }
                }
                nextDate = DateUtils.parseDateToStr("yyyy-MM", date) + "-" + cycleDate;

            }else {//如果指定每月几号小于今天所属几号，则取下个月的几号
                Date lastMonthDate = DateUtils.getLastMonthDate(DateUtils.parseDateToStr(YYYY_MM_DD_HH_MM_SS, date), 1);
                nextDate = DateUtils.parseDateToStr("yyyy-MM", lastMonthDate) + "-" + cycleDate;
            }
        }

        return nextDate + " " + beginDateTime;
    }

    /**
     * 指定日期字符串加/减指定月数
     * @param dataTime
     * @param months
     * @return
     * @throws ParseException
     */
    public static Date getLastMonthDate(String dataTime,int months) throws ParseException {
        //时间字符串转 LocalDate 类型
        LocalDate today = LocalDate.parse(dataTime, DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS));
        //当前月份+（-1）
        today = today.plusMonths(months);
        Instant instant = today.atTime(LocalTime.MIDNIGHT).atZone(ZoneId.systemDefault()).toInstant();
        Date date = Date.from(instant);
        return date;

    }

    /**
     * 获取日期列表中离当前日期最近的一个日期
     * @param dateList
     * @return
     */
    public static String getLatstDate(List<Date> dateList){

        final long now = System.currentTimeMillis();
        // Get date closest to "now"
        Date closest = Collections.min(dateList, new Comparator<Date>() {
            public int compare(Date d1, Date d2) {
                long diff1 = Math.abs(d1.getTime() - now);
                long diff2 = Math.abs(d2.getTime() - now);
                return Long.compare(diff1, diff2);
            }
        });
        return DateUtils.parseDateToStr(YYYY_MM_DD_HH_MM_SS, closest);
    }

}
